# Importing core models in order to enable "dumpdata"

from .core.apps.devices.models import *
from .core.apps.groups.models import *
from .core.apps.passepartout.models import *
from .core.apps.profiles.models import *
from .core.apps.services.models import *
from .core.apps.status.models import *
from .core.apps.events.models import *
