# Generated by Django 2.2.12 on 2020-07-03 09:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_sso_app', '0005_auto_20200519_1725'),
    ]

    operations = [
        migrations.DeleteModel(
            name='EmailAddress',
        ),
    ]
