{% extends "account/base.html" %}
{% load static %}
{% load i18n %}
{% load account socialaccount %}

{% block title %}User: {{ user.username }}{% endblock %}

{% block content %}
<div class="container container-full">
    <h1 class="text-center">{{APP_DOMAIN}}</h1>
</div>
{% endblock content %}