{% load i18n %}
{% get_current_language as LANGUAGE_CODE %}

<p class="app-links text-center">
    <a class="active" href="{% url 'home' %}">Home <span class="sr-only">(current)</span></a>
    {% if I18N_PATH_ENABLED %}
        <a class="" href="{% url 'about-'|add:LANGUAGE_CODE %}">About</a>
    {% else %}
        <a class="" href="{% url 'about' %}">About</a>
    {% endif %}

    {% if request.user.is_authenticated %}
    {# URL provided by django-allauth/account/urls.py #}
        <a class="" href="{% url 'profile' %}">{% trans "Profile" %}</a>

    {# URL provided by django-allauth/account/urls.py #}
        <a class="" href="{% url 'profile.update' %}">{% trans "Update Profile" %}</a>

    {# URL provided by django-allauth/account/urls.py #}
        <a class="" href="{% url 'account_email' %}">{% trans "Change E-mail" %}</a>

    {# URL provided by django-allauth/account/urls.py #}
        <a class="" href="{% url 'account_logout' %}">{% trans "Sign Out" %}</a>

    {% else %}

    {# URL provided by django-allauth/account/urls.py #}
        <a id="sign-up-link" class="" href="{% url 'account_signup' %}">{% trans "Sign Up" %}</a>

    {# URL provided by django-allauth/account/urls.py #}
        <a id="log-in-link" class="" href="{% url 'account_login' %}">{% trans "Sign In" %}</a>

    {% endif %}
</p>